// dialogs.cpp
// routines for most of the dialog windows
// v0.84 (7.3.2002)
// (c) 2002 Andreas Krueger
//
// Please, if you change anything in this code,
// mark the end of those lines with '// *+* your name *+* ' 
// (so that I can find it) and send the code 
// to cye-code@AndreasKrueger.de
//
// last change: 
//          by:
 
#include "dialogs.h"
#include "constants.h"
#include "storage.h"
#include "helpers.h"
#include "exceptions.h"
#include "invitation.h"
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlayout.h>

Dialogs::Dialogs ( ClientStorage * theDataPointer, QWidget *parent, const char *name )
		: QWidget(parent, name)
{
	this->resize(0,0); // it doesn't show on the mainwindow

	theData=theDataPointer;

    About= new QMessageBox ( "About (CYE) CountYourEuro!",
		            tr(ABOUTTEXT).arg(PROGRAMVERSION/100.0).arg(__DATE__),
                    QMessageBox::Information,
                    QMessageBox::Yes | QMessageBox::Escape | QMessageBox::Default,
                    QMessageBox::No,
                    QMessageBox::Cancel );
		About->setButtonText( QMessageBox::Yes, "OK" );
		About->setButtonText( QMessageBox::No, "WWW:\nProject Homepage" );
		About->setButtonText( QMessageBox::Cancel, "WWW:\nDownload new version" );

	/*QPushButton buttonQuitAbout("quit",About,"dialogAbout Quit");
	About->resize(300,200);
	About->setCaption("CYE - About this project");
	connect (&buttonQuitAbout, SIGNAL(clicked()), About, SLOT( hide()) );
	*/

	// init the Settings Dialog:
	Settings= new SettingsDialog (this, "SettingsDialogWindow", true,
		WStyle_Customize | WStyle_NormalBorder | WStyle_Title  );
	
	connect(Settings->OldSettingsButton, SIGNAL(clicked()),
		this, SLOT(SettingsResetToOldValues()) );
	connect (Settings->Country, SIGNAL(activated(int)),
		this, SLOT(SettingsCountryChanged(int)) );

	//
	connect( Settings->several, SIGNAL(clicked()),
		this, SLOT(execSeveralRecommenderNumbersDialog()) );

	// prevent illogical Year / Month combinations:
	connect (Settings->MixdateYear, SIGNAL(activated(int)),
		this, SLOT(MixdateYearChanged(int)) );
	connect (Settings->ArrivaldateYear, SIGNAL(activated(int)),
		this, SLOT(ArrivaldateYearChanged(int)) );
	connect (Settings->MixdateMonth, SIGNAL(activated(int)),
		this, SLOT(MixdateMonthChanged(int)) );
	connect (Settings->ArrivaldateMonth, SIGNAL(activated(int)),
		this, SLOT(ArrivaldateMonthChanged(int)) );

	createSeveralRecommenderNumbersDialog();

	SettingsCopyFromStorage(*theData);
	SettingsCreateObservationEvents();

	Clientwindow=new QDialog(this,"client",true);
	theClient = new Client( theData, this, Clientwindow, "clientobject");
	QGridLayout *cg=new QGridLayout(Clientwindow, 1,1,3);
	cg->addWidget(theClient, 0,0);
	Clientwindow->resize(400,550);
	connect (theClient, SIGNAL(Event(QString)), this, SIGNAL(Event(QString)) );

	InvitationsWindow=new QDialog(this,"invitationwindow",true);
	InvitationsWindow->setCaption("Send CYE invitations to your friends");
	theInvitations=new Invitations(InvitationsWindow, "invitations");
	QGridLayout *icg=new QGridLayout(InvitationsWindow, 1,2,3);
	icg->addWidget(theInvitations, 0,0);
	connect (theInvitations, SIGNAL(ok()), InvitationsWindow, SLOT(accept()) );
	connect (theInvitations, SIGNAL(Event(QString)), this, SIGNAL(Event(QString)) );
}

void Dialogs::setPicsAlsoInInvitations(QPixmap pics[NumCoins][NumCountries+1], QSize theMaxSize){
	theInvitations->theCoinShow->setPics(pics, theMaxSize);
	// TODO: let SlideShow have PointerToPics rather than Pics,
	//       so that they can be recycled without copying
	theInvitations->theCoinShow->changeCoinSide(); 
}

void Dialogs::CopyFromThenExecInvitationWindowAndCopyTo(QValueList<InvitedPerson > &lip){
	theInvitations->invited->clear();
	fromValueListToListView(lip, theInvitations->invited);
	theInvitations->setFavouriteCoin(theData->FavouriteCoin);
	InvitationsWindow->exec();
	theData->FavouriteCoin=theInvitations->theCoinShow->actualCoinNumber();
	fromListViewToValueList(theInvitations->invited, lip);
	// return lip
}

bool Dialogs::openClientForUploadAndIfSuccessStoreEverything(){
	theClient->initClient(QString(theData->ServerAddress.c_str()) , theData->ServerPort);
	if(Clientwindow->exec()){
		emit Event("ClientWindowClosedProperlyAfterUpload");
		// Todo: react to "reject()" if the data could not be uploaded
		// this here only quits if "accept()"
		this->appendToArchiveFile(*theData, ARCHIVEFILENAME);
		theData->LastUpload=QDateTime::currentDateTime();
		theData->SettingsFlags &= (65535-256); // reset changed-settings bit
		theData->NumberOfUploads++;
		theData->NumberOfStartsSinceLastUpload=0;
		theData->Sessions.clear();
		theData->NumFullPurses=0;
		theData->Message="";
		setAllUploadedTrue(theData->InvitationList);
		theData->saveToDisk(FILENAME);
		return true;
	}
	else{
		emit Event("ClientWindowAborted");
		return false;
	}
}

#include<qmultilineedit.h>
#include<qlayout.h>
#include <math.h>

void Dialogs::goodbyeShow(SlideShow *SlideShowPointer){
	ss=SlideShowPointer;
	byebye= new QDialog(this, "byebye", true);
	byebye->setCaption("CYE says thank you");

	QGridLayout *grid=new QGridLayout( byebye, 1,2,7);
	grid->addColSpacing(1,350);
	QMultiLineEdit *text= new QMultiLineEdit (byebye);
	text->setGeometry( QRect( 124, 151, 335, 105 ) );
	text->setText("");
	text->append("Thanks a lot.");
	QString temp = "Your data has been saved to %1.";
	temp=temp.arg(FILENAME);
	text->append(temp);
	text->append("\nSee you next time...");
	text->append(" ");
	text->append(" ");
	text->append(" ");
	srand(QTime::currentTime().msecsTo(QTime(0,0,0)));
	byecountry=1+(char)floor(NumCountries*((float)rand()/(float)RAND_MAX));
	if (byecountry>=(char)NumCountries) byecountry=(char)NumCountries;
	
	text->append("Today featuring ...");
	text->append("\t\t\t"+countrynamesWords[byecountry]);

	grid->addWidget(text, 0,1);
	byepic=new QLabel(byebye);
	
	byepic->setPixmap(ss->centeredCoin(NumCoins-1, byecountry));
	grid->addWidget(byepic, 0,0);

	byecoin=0;
	QTimer *flipper= new QTimer(this);
	connect(flipper, SIGNAL(timeout()), this, SLOT(quit_flipToNextPic()) );
	QTimer::singleShot(3000, qApp, SLOT(quit()) );
	flipper->start(80);
	byebye->exec();
}

void Dialogs::quit_flipToNextPic(){
	byecoin++; 
	if (byecoin>=(char)NumCoins) {
		byecoin=0;
	}
	byepic->setPixmap(ss->centeredCoin(byecoin, byecountry));
}

void Dialogs::createUserObservationWindow(Usage & Observation){
	UserObservation=new QDialog ( this, "observation", false);
	UserObservation->resize( 520, 420 ); 
	UserObservation->setName( "User Observation Results" );
	QGridLayout *grid=new QGridLayout(UserObservation,2,1,5);

	SessionEventsLV= new QListView (UserObservation, "session events listview");
	SessionEventsLV->setGeometry( QRect( 10, 10, 250, 400 ) ); 
	AllEventsLV= new QListView(UserObservation, "all events listview");
	AllEventsLV->setGeometry( QRect( 260, 10, 250, 400 ) );
	
	grid->addWidget(SessionEventsLV,0,0);
	grid->addWidget(AllEventsLV,0,1);

	Observation.PrepareSessionListView(SessionEventsLV);
	Observation.PrepareAllEventsListView(AllEventsLV);

	UserObservation->show();
}

void Dialogs::updateEventsViewerWindow(Usage & Observation){
	Observation.UpdateSessionListView(SessionEventsLV);
	Observation.UpdateAllEventsListView(AllEventsLV);
}

#include <qmessagebox.h>
#include <qapplication.h>


void Dialogs::startSettingsDialog(bool startWithAlertBox, bool firstTime){
	SettingsCopyFromStorage(*theData);

	int filledInResult= 1;
	bool savedError=false;

	while(filledInResult == 1){
		int result;
		if (startWithAlertBox) {
			filledInResult=SettingsValidationOK(firstTime);
			startWithAlertBox=false;
		}
		else{
			result= Settings->exec();
			filledInResult=SettingsValidationOK(firstTime);
		}
		theData->setNecessarySettingsOK(false); // necessary stuff was NOT filled in

		if (filledInResult == 0){
			SettingsCopyToStorage(*theData);
			theData->setNecessarySettingsOK(true); // necessary stuff was filled in

			savedError=theData->saveToDisk(FILENAME);
			if (savedError!=0) {
				switch( QMessageBox::warning( this, "CYE cannot save your settings",
						ERRORSAVING.arg(savedError).arg(FILENAME).arg(FILENAME),
						"OK, I try my best.\nPlease return to the settings.", 
						"No, discard my settings changes.", 0, 0, 1 ) 
						) {	
				case 0: 
					emit Event("SettingsWriteErrorDialogReturn");
					// Settings->OKButton->setDown(false);
					filledInResult=1;
				 break;
				default: 
					emit Event("SettingsWriteErrorDialogDiscard");
					filledInResult=0;
					break;  
				}
			}
			else { 
				Event("SettingsSaved");
			}
		}
		if (filledInResult == 2){
			emit Event("SettingsEndedWithUserWantsProgramTermination");
			SettingsCopyToStorage(*theData);
			emit Event("BrowseWhynot");
			openBrowser("http://www.AndreasKrueger.de/euro/discussion/whynot.html");
			emit QuitEvent((int) USER_DOESNT_WANT_TO_INPUT_NECESSARY_DATA);
		}
	}
	emit Event("SettingsDialogClosed");
}



#include <qtabwidget.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qapplication.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qlistbox.h>

bool Dialogs::commaSeparatedToListBox(QString str, int &errorpos, QListBox * LB){
	bool ok=true;
	QString oneWord, numbers="0123456789", separators=" , ";
	QChar c;
	errorpos=0; int end=str.length();
	Q_UINT32 rmn;

	if (str.isEmpty()) return true;
	else{
		do{ // for each word=number
			c=str[errorpos];
			while  (separators.contains(c) && (errorpos<(end-1)) ){
				errorpos++;
				c=str[errorpos];
			} 
			if (errorpos<end && !numbers.contains(c) ) 
				return false;
			else {
				oneWord="";
				do { // until this word=number ends
					oneWord += c ;
					errorpos++;
					c=str[errorpos];
				} while  (numbers.contains(c) && (errorpos<(end)) );
				rmn=oneWord.toInt( &ok, 10);
				if (!ok) return false;
				else LB->insertItem(oneWord);
			}
		} while (errorpos < end);
	}
	return true;
}
QString Dialogs::ListBoxToCommaSeparated(QListBox * LB){
	QString str="";
	for (uint i=0; i< LB->count() ; i++){
		str+=LB->text(i);
		str+=", ";
	}
	str=str.left(str.length()-2);
	return str;
}

bool Dialogs::ListBoxToValueList(QListBox *LB, QValueList<Q_UINT32> &intoTheList){
	bool ok=true, allOk=true; Q_UINT32 rmn;
	for (uint i=0; i< LB->count() ; i++){
		rmn=LB->text(i).toInt( &ok, 10);
		if (ok) 
			intoTheList.append(rmn);
		else 
			allOk=false;
	}
	return allOk;
}
void Dialogs::ValueListToListBox(QValueList<Q_UINT32> theList, QListBox *LB){
	QValueList<Q_UINT32>::Iterator it=theList.begin();
	QString number;
	for (; it!=theList.end(); it++){
		number.setNum(*it);
		LB->insertItem(number /* ,0 */ );
	}
}	



void Dialogs::createSeveralRecommenderNumbersDialog(){
	severalRecommendersDialog= new QDialog(this,"several RecommenderNumbers", true);
	QGridLayout *grid=new QGridLayout(severalRecommendersDialog, 4,3,5);
	QLabel *label1=new QLabel("Type in the membernumbers of those people who "
		"recommended this programm to you. Then later I can show you how far "
		"your influence has spread over Europe.", severalRecommendersDialog, "label");
	label1->setProperty( "alignment", int( QLabel::WordBreak | QLabel::AlignVCenter | QLabel::AlignCenter ) );
	errorLabel=new QLabel(severalRecommendersDialog);
	errorLabel->setProperty( "alignment", int( QLabel::WordBreak | QLabel::AlignVCenter | QLabel::AlignCenter ) );
	newRecommender=new QLineEdit(severalRecommendersDialog, "rmn");
	QPushButton *buttonAddRecommender=new QPushButton("&Add", severalRecommendersDialog, "button add rmn");
	recommendersLB=new QListBox(severalRecommendersDialog, "recommenders");
	QPushButton *buttonDelRecommender=new QPushButton("&Del", severalRecommendersDialog, "button del rmn");
	QPushButton *buttonOK=new QPushButton("O&K", severalRecommendersDialog, "button ok rmn");
	grid->addMultiCellWidget(label1, 0,0,0,2);
	grid->addMultiCellWidget(newRecommender, 1,1,0,1);
	grid->addWidget(buttonAddRecommender, 1,2);
	grid->addMultiCellWidget(recommendersLB, 2,3,0,1);
	grid->addWidget(buttonDelRecommender, 2,2);
	grid->addMultiCellWidget(errorLabel, 4,4,0,1);
	grid->addWidget(buttonOK, 4,2);
	grid->addRowSpacing(3,100);

	connect(buttonAddRecommender, SIGNAL(clicked()), this, SLOT(addNewRecommender()) );
	connect(buttonDelRecommender, SIGNAL(clicked()), this, SLOT(delRecommender()) );
	connect(buttonOK, SIGNAL(clicked()), severalRecommendersDialog, SLOT(accept()) );
}

void Dialogs::delRecommender(){
	int curr=recommendersLB->currentItem();
	if (curr>=0){
		recommendersLB->removeItem(curr);
		errorLabel->setText("");
		emit Event("SettingsRecommenderMembernumberSeveralDel");
	}
	else {
		errorLabel->setText("click the to be deleted first");
		emit Event("SettingsRecommenderMembernumberSeveralDelWrong");
	}
}
void Dialogs::addNewRecommender(){
	bool ok=true;
	QString number=newRecommender->text();
	number.toInt( &ok, 10);
	if (ok){
		errorLabel->setText("");
		recommendersLB->insertItem(number,0);
		newRecommender->setText("");
		emit Event("SettingsRecommenderMembernumberSeveralAdd");
	}
	else{ 
		errorLabel->setText(number+" is no number.");
		emit Event("SettingsRecommenderMembernumberSeveralAddWrong");
	}
}

void Dialogs::execSeveralRecommenderNumbersDialog(){
	emit Event("SettingsRecommenderMembernumberSeveral");
	QString recommenders; bool ok=true; 
	recommenders=Settings->RecommenderMembernumber->text();
	recommendersLB->clear();
	int pos; ok=commaSeparatedToListBox(recommenders, pos, recommendersLB);
	if (!ok) {
		QMessageBox::information( this, "Wrong Membernumber Format", 
				tr("For the recommenders' membernumbers \"%1\" is a wrong entry."
				"\nThey have to be (comma-separated) NUMBERs.")
				.arg(recommenders.mid(pos)) );
				emit Event("SettingsMembernumberNonNumberFormat");
	}
	else{
		newRecommender->setText("");
		severalRecommendersDialog->exec();
		emit Event("SettingsRecommenderMembernumberSeveralEnd");
		Settings->RecommenderMembernumber->setText(ListBoxToCommaSeparated(recommendersLB));
	}
}


int Dialogs::SettingsValidationOK(bool firstTime){

	// are the 3 necessary informations filled in?
	bool missing=false;
	if (Settings->Country->currentText()==""){
		missing=true;
		Settings->LabelCountry->setPalette( QPalette( QColor(80, 80, 255) ) );
		emit Event("SettingsCountryMissing");
	}
	else 	Settings->LabelCountry->unsetPalette();

	if (Settings->PostalCode->text().length()<2){
		missing=true;
		Settings->LabelPostalCode->setPalette( QPalette( QColor(80, 80, 255) ) );
		emit Event("SettingsPostalCodeMissing");
	} 
	else 	Settings->LabelPostalCode->unsetPalette();

	if (Settings->Population->currentText()==""){
		missing=true;
		Settings->LabelPopulation->setPalette( QPalette( QColor(80, 80, 255) ) );
		emit Event("SettingsPopulationMissing");
	}
	else 	Settings->LabelPopulation->unsetPalette();

	bool contract=true; bool opt_in=true;
	if (! Settings->paragraph1->isChecked()){
		contract=false;
		Settings->paragraph1->setPalette( QPalette( QColor(80, 80, 255) ) );
		emit Event("SettingsParagraph1NotChecked");
	} else Settings->paragraph1->unsetPalette();
	if (! Settings->paragraph2->isChecked()){
		opt_in=false;
		Settings->paragraph2->setPalette( QPalette( QColor(80, 255, 80) ) );
		emit Event("SettingsParagraph2NotChecked");
	} else Settings->paragraph2->unsetPalette();

	QString text;
	if (firstTime)
		text="Before you can upload your data, you are asked \nto fill in some information about you.\n\n";
	else 
		text="Your CYE Settings are not complete!\n\n";

	if (missing) {
		text += "Please fill in where you are!"
		        "\nThe Euro diffusion depends on the location.\n";
	}
	if (missing && !contract) text+="\nMoreover:\n";
	if (!contract) {
		text+= "This program is part of a scientific research project.\n";
		if (firstTime)
			text+="To ensure data quality, you are asked to sign 1 of the contract";
		else text+="If you don't sign paragraph 1 of the contract,\n"
		           "you can't participate in this project, sorry.\n";
	}

	if (!opt_in) {
		text+= "\nOptional:"
			   "\nYou could enable us to optimize this and other programs"
			   "\nby research on ergonomics. Therefore, it would be very nice,"
			   "\nif you allowed us to log the usage by also signing paragraph 2.";
	}

	bool quitApp=false;
	QString title="CYE Settings not complete";
	if (firstTime){
		title="Please fill in necessary information";
	}
	if (missing || !contract){
		switch( QMessageBox::warning( this, title, text,
							"OK, I do", "No, I don't", 0, 0, 1 ) )
		{
			case 0: 
				emit Event("SettingsWantToInputMissingData");
			 break;
			case 1: 
				emit Event("SettingsDontWantToInputMissingData");
				switch( QMessageBox::warning( this, "Participation endangered", 
					                 "Then the program would be terminated now\n"
									 "loosing your data...!",
							"OK, I want to participate", "Quit, I don't participate", 
							0, 0, 1 ) )
				{
				case 0: 
					emit Event("SettingsMissingDataPersuadedContinue");
					break;
				case 1:
					emit Event("SettingsMissingDataNotPersuadedQuit");
					quitApp=true;
					break;
				}
			break;
		}
		Settings->OKButton->setDown(false);
		Settings->TabWidget2->setCurrentPage(0);
	}

	// don't accept rubbish in RecommenderUsernumber:
	QString recommenders; bool ok=true; 
	recommenders=Settings->RecommenderMembernumber->text();
	recommendersLB->clear();
	int pos=0;
	ok=commaSeparatedToListBox(recommenders, pos, recommendersLB);
	if (!ok) {
		QMessageBox::information( this, "Wrong Membernumber Format", 
				tr("For the recommenders' membernumbers \"%1\" is a wrong entry."
				"\nThey have to be (comma-separated) NUMBERs.").arg(recommenders.mid(pos)) );
				emit Event("SettingsMembernumberNonNumberFormat");
	}
	recommenders=ListBoxToCommaSeparated(recommendersLB);
	Settings->RecommenderMembernumber->setText(recommenders);

	return (quitApp)?2:(missing || !contract || !ok)?1:0;
}


void Dialogs::SettingsResetToOldValues(){
	SettingsCopyFromStorage(*theData);
}

void Dialogs::SettingsCountryChanged(int index){
	if (index>(int) NumCountries)	{
		Settings->LabelPostalCode->setText(" &Country, City");
		QToolTip::remove(Settings->PostalCode);
		QToolTip::add(Settings->PostalCode, 
			"Please name your Country and City");
	}
	else {
		Settings->LabelPostalCode->setText(" Postal &Code");
		QToolTip::remove(Settings->PostalCode);
		QToolTip::add(Settings->PostalCode, 
			"The postal code of your city (at least 2 digits).");
	}
}

void Dialogs::MixdateYearChanged(int dummy){
	doNothing(dummy); // don't use it here
	if (Settings->MixdateYear->currentText()=="Never"
		|| Settings->MixdateYear->currentText().contains("-"))
	{	Settings->MixdateMonth->setEnabled(false);
		Settings->MixdateMonth->setCurrentItem(0);
	}
	else Settings->MixdateMonth->setEnabled(true);

	if (Settings->MixdateYear->currentItem()==0)
		MixdateMonthChanged();
}
void Dialogs::ArrivaldateYearChanged(int dummy){
	doNothing(dummy); // don't use it here
	if (Settings->ArrivaldateYear->currentText()=="Never"
		|| Settings->ArrivaldateYear->currentText().contains("-"))
	{	Settings->ArrivaldateMonth->setEnabled(false);
		Settings->ArrivaldateMonth->setCurrentItem(0);
	}
	else Settings->ArrivaldateMonth->setEnabled(true);

	if (Settings->ArrivaldateYear->currentItem()==0)
		ArrivaldateMonthChanged();
}

void Dialogs::MixdateMonthChanged(int dummy){
	doNothing(dummy); // don't use it here
	if (Settings->MixdateMonth->currentItem()!=0
		&& Settings->MixdateYear->currentItem()==0)
	{	int year_index;
		year_index=maxOf2(1,QDate::currentDate().year() - 2002 + 1);
		year_index= (year_index<10)?year_index:(year_index<20)? 10:11;
		Settings->MixdateYear->setCurrentItem(year_index);
		MixdateYearChanged();
	}
}
void Dialogs::ArrivaldateMonthChanged(int dummy){
	doNothing(dummy); // don't use it here
	if (Settings->ArrivaldateMonth->currentItem()!=0
		&& Settings->ArrivaldateYear->currentItem()==0)
	{	int year_index;
		year_index=maxOf2(1,QDate::currentDate().year() - 2002 + 1);
		year_index= (year_index<10)?year_index:(year_index<20)? 10:11;
		Settings->ArrivaldateYear->setCurrentItem(year_index);
		ArrivaldateYearChanged();
	}
}


#include <qcheckbox.h>
#include <qglobal.h>
#include <qradiobutton.h>
#include <qdatetime.h>
#include <qmultilineedit.h>

void Dialogs::SettingsCopyToStorage(ClientStorage & D){
	D.SettingsFlags |= 256;  // enable the Flag that the Settings might have changed

	bool ok; QString temp;
	const SettingsDialog & S = *Settings;

    D.Country=S.Country->currentItem();
	D.PostalCode=S.PostalCode->text();
    D.CityPopulation=S.Population->currentItem();
	
	if (S.paragraph1->isChecked() )
		D.SettingsFlags |= 64;
	else 	D.SettingsFlags &= (65535-64);
	if (S.paragraph2->isChecked())
		D.SettingsFlags |= 128;
	else 	D.SettingsFlags &= (65535-128);

	D.RecommenderName=S.RecommenderName->text();
	D.RecommenderMembernumbers.clear();
	ok=ListBoxToValueList(recommendersLB, D.RecommenderMembernumbers);
	if (!ok) qDebug("strange: not valid Listbox in void Dialogs::SettingsCopyToStorage(ClientStorage & D)");
	
	D.eMailFlags=0;
	if (S.WantsReminderEmail->isChecked()) D.eMailFlags=8;

	if (S.weekly->isChecked()){	
		D.eMailFlags+=4;}
	else if (S.bimonthly->isChecked()) 
			 D.eMailFlags+=2;
		 else if (S.monthly->isChecked()) 
				  D.eMailFlags+=1;
	if (S.WantsRegularInformations->isChecked()) D.eMailFlags += 16;
    if (S.WantsToHelp->isChecked() ) D.eMailFlags += 32;
	if (S.WantsToAnalyze->isChecked()) D.eMailFlags +=64;

    D.eMailAddress=S.email->text();
    
	D.GuessWhenMixdateYear    = S.MixdateYear->currentItem();
	D.GuessWhenMixdateMonth   = S.MixdateMonth->currentItem();
	D.GuessWhenArrivaldateYear= S.ArrivaldateYear->currentItem();
	D.GuessWhenArrivaldateMonth=S.ArrivaldateMonth->currentItem();

	D.Message=S.Message->text();
	
	if (S.ServiceTabOn->isChecked())  	D.SettingsFlags |= 1;
	   else						  		D.SettingsFlags &= (65535-1);

	if (S.OpinionTabOn->isChecked())  	D.SettingsFlags |= 2;
	   else						  		D.SettingsFlags &= (65535-2);
}

#include <qbuttongroup.h>
#include <qbrush.h>
#include <qlcdnumber.h>

void Dialogs::SettingsCopyFromStorage(const ClientStorage & D){
	QString temp;
	SettingsDialog & S = *Settings;

    S.Country->setCurrentItem(D.Country);
	S.PostalCode->setText(D.PostalCode);
    S.Population->setCurrentItem(D.CityPopulation);

	S.paragraph1->setChecked(D.SettingsFlags & 64);
	S.paragraph2->setChecked(D.SettingsFlags & 128);
		
	recommendersLB->clear();
	if (D.RecommenderMembernumbers.count()==0){
		S.RecommenderMembernumber->setText("");
	}
	else{
		ValueListToListBox(D.RecommenderMembernumbers, recommendersLB);
		S.RecommenderMembernumber->setText(ListBoxToCommaSeparated(recommendersLB));
	} 
	S.RecommenderName->setText(D.RecommenderName);

	S.WantsReminderEmail->setChecked( D.eMailFlags & 8 );

	S.weekly->setChecked   ( D.eMailFlags & 4 );
	S.bimonthly->setChecked( D.eMailFlags & 2 );
	S.monthly->setChecked  ( D.eMailFlags & 1 );

	S.WantsRegularInformations->setChecked( D.eMailFlags & 16 );
    S.WantsToHelp->setChecked( D.eMailFlags & 32 );
	S.WantsToAnalyze->setChecked( D.eMailFlags & 64 );

    S.email->setText(D.eMailAddress.c_str());

	S.MixdateYear->setCurrentItem(D.GuessWhenMixdateYear);
	S.MixdateMonth->setCurrentItem(D.GuessWhenMixdateMonth);
	S.ArrivaldateYear->setCurrentItem(D.GuessWhenArrivaldateYear);
	S.ArrivaldateMonth->setCurrentItem(D.GuessWhenArrivaldateMonth);
	MixdateYearChanged();
	ArrivaldateYearChanged();

	S.Message->setText(D.Message);
	S.ServiceTabOn->setChecked( D.SettingsFlags & 1 );
	S.OpinionTabOn->setChecked( D.SettingsFlags & 2 );

	if (D.Welcome=="") S.Welcome->setPaper(QBrush(lightGray));
	else{ 
		S.Welcome->setPaper(QBrush(white));
		S.Welcome->setText(D.Welcome.c_str());
	}
	if (D.Membernumber==0) S.MemberNumber->display("");
	else {
		S.MemberNumber->display((int)D.Membernumber);
		S.MemberNumber->setSegmentStyle(QLCDNumber::Filled);
		S.MemberPanel->setTitle("Your membership in the CYE-project");
	}
	if (D.MemberSince == QDateTime (QDate(2002,2,1),QTime(0,0,0)) ){
		S.MemberSince->setText("");
		S.Welcome->setPaper(QBrush(lightGray));
	} else {
		S.MemberSince->setText(D.MemberSince.toString());
		S.Welcome->setPaper(QBrush(white));
	}

}

void Dialogs::appendToArchiveFile(ClientStorage theData, QString filename){
	ClientStorage archive; int ok=0;
	try{
		ok=archive.loadFromDisk(filename);
	}
	catch (myException e){
		switch(QMessageBox::warning(this, "CYE: archive file error", errorMessageAndSolution(e.errnum),
			"Ignore\nsaved data.", 
			"Quit now\n(loosing new data of today!)", 0, 0, 1 ) )
		{	case 0: 
				emit Event("BadArchivefileOverwrite");
				archive.setDefaultValues();
		  	  break;
			default:
				throw;
			  break;
		}
	}

	if (ok!=0)
		archive.setDefaultValues();  // then there wasn't an archive yet
	archive.MergeWithNewSessions(theData);
saveloop:
	ok=archive.saveToDisk(filename);
	if (ok!=0)
	{
		switch( QMessageBox::warning( this, "CYE cannot save your archive",
                    ERRORSAVING.arg(ok).arg(filename).arg(filename),
					"OK, I try my best.\nPlease try again", 
					"No, quit anyway. I don't need an archive.", 0, 0, 1 ) )
		{	case 0: 
				//emit Event("ArchiveWriteErrorDialogReturn");
				goto saveloop;
				break;
			default: 
				//emit Event("ArchiveWriteErrorDialogDiscard");
				break;
		}
	}
}



#include <qdialog.h>
#include <qpushbutton.h>

void Dialogs::TableOfErrorsAndRecommendations(QWidget *parent){
	QDialog *W=new QDialog (parent);
	QListView *LV=new QListView(W);
	QPushButton *quit=new QPushButton("quit", W);
	connect (quit, SIGNAL(clicked()), W, SLOT(accept()) );
	QGridLayout *g=new QGridLayout(W, 2,1,5);
	g->addWidget(LV, 0,0);
	g->addWidget(quit, 1,0);

	LV->clear();
	LV->setRootIsDecorated(true);
    LV->setSorting( -1 ); 
    LV->addColumn( "num" );
    LV->addColumn( "errror" );
	LV->addColumn( "suggestion" );

	QString num;
	for (int i=0; i<=numErrorMsgs; i++){
		(void) new QListViewItem(LV, num.setNum(i), errormessage[i], errorhelp[i]);
	}

	W->show();
}



// these events are raised when anything happens to the settings dialog

void Dialogs::SettingsCreateObservationEvents(){

	SettingsDialog & S = *Settings;

    connect(S.Country, SIGNAL(activated(int)), this, SLOT(eventCountry(int)) );
    connect(S.PostalCode, SIGNAL(textChanged(const QString &)), 
		this, SLOT(eventPostalCode(const QString&)) );
	connect(S.Population, SIGNAL(activated(int)), this, SLOT(eventPopulation(int)) );
	connect(S.paragraph1, SIGNAL(toggled(bool)), this, SLOT(eventParagraph1(bool)) );
	connect(S.paragraph2, SIGNAL(toggled(bool)), this, SLOT(eventParagraph2(bool)) );
	connect(S.RecommenderMembernumber, SIGNAL(textChanged(const QString&)), 
		this, SLOT(eventRecommenderMembernumber(const QString& )) );
	connect(S.RecommenderName, SIGNAL(textChanged(const QString& )), 
		this, SLOT(eventRecommenderName(const QString&)) );
	connect(S.WantsReminderEmail, SIGNAL(toggled(bool)), 
		this, SLOT(eventWantsReminderEmail(bool)) );
	connect(S.weekly, SIGNAL(toggled(bool)), this, SLOT(eventWeekly(bool)) );
	connect(S.bimonthly, SIGNAL(toggled(bool)), this, SLOT(eventBiMonthly(bool)) );
	connect(S.monthly, SIGNAL(toggled(bool)), this, SLOT(eventMonthly(bool)) );
	connect(S.WantsRegularInformations, SIGNAL(toggled(bool)), 
		this, SLOT(eventWantsRegularInformations(bool)) );
	connect(S.WantsToHelp, SIGNAL(toggled(bool)), this, SLOT(eventWantsToHelp(bool)) );
	connect(S.WantsToAnalyze, SIGNAL(toggled(bool)), this, SLOT(eventWantsToAnalyze(bool)) );
	connect(S.email, SIGNAL(textChanged(const QString& )), 
		this, SLOT(eventEmail(const QString&)) );
	connect(S.MixdateYear, SIGNAL(activated(int)), this, SLOT(eventMixdateYear(int)) );
	connect(S.MixdateMonth, SIGNAL(activated(int)), this, SLOT(eventMixdateMonth(int)) );
	connect(S.ArrivaldateYear, SIGNAL(activated(int)), this, SLOT(eventArrivaldateYear(int)) );
	connect(S.ArrivaldateMonth, SIGNAL(activated(int)), this, SLOT(eventArrivaldateMonth(int)) );
	connect(S.Message, SIGNAL(textChanged()), 
		this, SLOT(eventMessage()) );
	connect(S.ServiceTabOn, SIGNAL(toggled(bool)), this, SLOT(eventServiceSwitchToggled(bool)) );
	connect(S.OpinionTabOn, SIGNAL(toggled(bool)), this, SLOT(eventOpinionSwitchToggled(bool)) );
	connect(S.OldSettingsButton, SIGNAL(clicked()), this, SLOT(eventOldSettingsButton()) );
	connect(S.OKButton, SIGNAL(clicked()), this, SLOT(eventSaveButton()) );
	connect(S.TabWidget2, SIGNAL(currentChanged(QWidget *)), this, SLOT(eventTabChanged(QWidget *)) );
}

void Dialogs::eventCountry(int dummy){ doNothing(dummy); emit Event("SettingsCountryChanged");}
void Dialogs::eventPostalCode(const QString &dummy){ doNothing(dummy); emit Event("SettingsPostalCodeChanged");}
void Dialogs::eventPopulation(int dummy) { doNothing(dummy); emit Event("SettingsPopulationChanged");}
void Dialogs::eventParagraph1(bool dummy){ doNothing(dummy); emit Event("SettingsParagraph1Toggled");}
void Dialogs::eventParagraph2(bool dummy){ doNothing(dummy); emit Event("SettingsParagraph2Toggled");}
void Dialogs::eventRecommenderMembernumber(const QString& dummy){ doNothing(dummy); emit Event("SettingsRecommenderMembernumberChanged");}
void Dialogs::eventRecommenderName(const QString& dummy){ doNothing(dummy); emit Event("SettingsRecommenderNameChanged");}
void Dialogs::eventWantsReminderEmail(bool dummy){ doNothing(dummy); emit Event("SettingsWantsReminderEmailToggled");}
void Dialogs::eventWeekly(bool on){if (on) emit Event("SettingsWeeklyON");}
void Dialogs::eventBiMonthly(bool on){if (on) emit Event("SettingsBiMonthlyON");}
void Dialogs::eventMonthly(bool on){if (on) emit Event("SettingsMonthlyON");}
void Dialogs::eventWantsRegularInformations(bool dummy){doNothing(dummy); emit Event("SettingsWantsRegularInformationsToggled");}
void Dialogs::eventWantsToHelp(bool dummy){doNothing(dummy); emit Event("SettingsWantsToHelpToggled");}
void Dialogs::eventWantsToAnalyze(bool dummy){doNothing(dummy); emit Event("SettingsWantsToAnalyzeToggled");}
void Dialogs::eventEmail(const QString& dummy){doNothing(dummy); emit Event("SettingsEMailChanged");}
void Dialogs::eventMixdateYear(int dummy){doNothing(dummy); emit Event("SettingsMixdateYearChanged");}
void Dialogs::eventMixdateMonth(int dummy){doNothing(dummy); emit Event("SettingsMixdateMonthChanged");}
void Dialogs::eventArrivaldateYear(int dummy){doNothing(dummy); emit Event("SettingsArrivaldateYearChanged");}
void Dialogs::eventArrivaldateMonth(int dummy){doNothing(dummy); emit Event("SettingsArrivaldateMonthChanged");}
void Dialogs::eventMessage(){emit Event("SettingsMessageChanged");}
void Dialogs::eventServiceSwitchToggled(bool dummy){doNothing(dummy); emit Event("SettingsServiceSwitchToggled");}
void Dialogs::eventOpinionSwitchToggled(bool dummy){doNothing(dummy); emit Event("SettingsOpinionSwitchToggled");}
void Dialogs::eventOldSettingsButton(){emit Event("SettingsOldSettingsButtonPressed");}
void Dialogs::eventSaveButton(){emit Event("SettingsSaveButtonPressed");}

void Dialogs::eventTabChanged(QWidget *){
	QString temp; temp.setNum(Settings->TabWidget2->currentPageIndex());
	emit Event("SettingsTabChangedTo"+temp);
}


// these routines extract the tables from the SettingsDialog made with QtDesigner
#include <strstream>
using std::ostrstream;

char* Dialogs::giveAllMemberCountriesAsTextTupels(char* thirdcolumn){
	SettingsDialog SD;
	ostrstream ss;
	int i;
	for (i=0; i<SD.Country->count();i++){ 
		ss<<"("<<i<<",'"<<SD.Country->text(i).latin1()<<"'"<<thirdcolumn<<"),";
	}
	ss.seekp(ss.tellp()-(long)1); // cut out the last comma
	ss<<'\0';
	// qDebug("%s",ss.str());

	return ss.str();
}

char* Dialogs::giveAllCitySizesAsTextTupels(char* thirdcolumn){
	SettingsDialog SD;
	ostrstream ss;
	int i;
	for (i=0; i<SD.Population->count();i++){ 
		ss<<"("<<i<<",'"<<SD.Population->text(i).latin1()<<"'"<<thirdcolumn<<"),";
	}
	ss.seekp(ss.tellp()-(long)1); // cut out the last comma
	ss<<'\0';
	// qDebug("%s",ss.str());

	return ss.str();
}


